import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;

/**
 * Undo ¸뤿Υ饹
 */

public class Undo {

	public static final int DO_NOTHING   = 0;
	public static final int ADD_NOTE     = 1;
	public static final int REMOVE_NOTE  = 2;
	public static final int DOUBLE_NOTE  = 3;
	public static final int HALVE_NOTE   = 4;
	public static final int ADD_DOT      = 5;
	public static final int REMOVE_DOT   = 6;
	public static final int MODIFY_NOTE  = 7;
	public static final int BEAM_NOTE    = 8;
	public static final int UNBEAM_NOTE  = 9;

	private LinkedList undoObjects = new LinkedList();
	private Button undoButton;

	Undo(Button ub) {
		undoButton = ub;
		undoButton.setEnabled(false);
		undoObjects.clear();
	}

	public void clear() {
		undoObjects.clear();
		undoButton.setEnabled(false);
	}

	public boolean canUndo() {
		return (!undoObjects.isEmpty());
	}

	public void undo() {
		UndoObject undoObject = (UndoObject)undoObjects.removeFirst();
		undoObject.undo();
		undoButton.setEnabled(canUndo());
	}

	//----------------------------------------------------------------------

	public void makeUndo(int t, Field f, Note n) {
		LinkedList ns = new LinkedList();
		ns.add(n);
		undoObjects.addFirst(new UndoObject(t, null, f, ns));
		undoButton.setEnabled(true);
	}

	public void makeUndo(int t, Field f, LinkedList ns) {
		undoObjects.addFirst(new UndoObject(t, null, f, ns));
		undoButton.setEnabled(true);
	}

	public void makeUndo(int t, int v[], Field f, Note n) {
		LinkedList ns = new LinkedList();
		ns.add(n);
		undoObjects.addFirst(new UndoObject(t, v, f, ns));
		undoButton.setEnabled(true);
	}

	public void makeUndo(int t, int v[], Field f, LinkedList ns) {
		undoObjects.addFirst(new UndoObject(t, v, f, ns));
		undoButton.setEnabled(true);
	}

}

//----------------------------------------------------------------------------------------------------

class UndoObject {

	int type;
	int value[];
	Field field;
	LinkedList notes;

	UndoObject(int t, int v[], Field f, LinkedList ns) {
		type = t;
		value = v;
		field = f;
		notes = ns;
	}

	void undo() {
		ListIterator it = notes.listIterator();
		switch (type) {
		case Undo.DO_NOTHING:
			break;
		case Undo.ADD_NOTE:
			while (it.hasNext()) {
				Note note = (Note)it.next();
				field.addNoteWithoutUndo(note);
			}
			break;
		case Undo.REMOVE_NOTE:
			while (it.hasNext()) {
				Note note = (Note)it.next();
				field.removeNoteWithoutUndo(note);
			}
			break;
		case Undo.DOUBLE_NOTE:
			while (it.hasNext()) {
				Note note = (Note)it.next();
				note.doubleNoteWithoutUndo();
			}
			break;
		case Undo.HALVE_NOTE:
			while (it.hasNext()) {
				Note note = (Note)it.next();
				note.halveNoteWithoutUndo();
			}
			break;
		case Undo.ADD_DOT:
			while (it.hasNext()) {
				Primitive note = (Primitive)it.next();
				note.addDotWithoutUndo();
			}
			break;
		case Undo.REMOVE_DOT:
			while (it.hasNext()) {
				Primitive note = (Primitive)it.next();
				note.removeDotWithoutUndo();
			}
			break;
		case Undo.MODIFY_NOTE:
			// value Ȥ
			// value[0] : numDot
			// value[1] : modification
			// value[2] : staccato Ȥ
			while (it.hasNext()) {
				Primitive n = (Primitive)it.next();
				if (n instanceof BasicNote) {
					BasicNote note = (BasicNote)n;
					note.numDot       = value[0];
					note.modification = value[1];
					note.staccato     = value[2];
				} else if (n instanceof Rest) {
					Rest note = (Rest)n;
					note.numDot = value[0];
				} else {
					Main.failwith("UndoObject : undo");
				}
			}
			break;
		case Undo.BEAM_NOTE:
			field.beamNoteWithoutUndo(notes);
			break;
		case Undo.UNBEAM_NOTE:
			while (it.hasNext()) {
				BeamedNote bn = (BeamedNote)it.next();
				field.unbeamNoteWithoutUndo(bn);
			}
			break;
		}
	}
}
